// 632_634_COMMAND_LINE_TEST.cpp
// Ultra-simple 632 / 634 command-line communications example

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "typedefs.h"
#include "serial.h"

#define MAXLINE 256
#define CMD_FADER 1
#define CMD_CONTRAST 2

struct Entry
{
	char 	command;
	int 	command_param;
	char	LCD_number;
	char	line1[MAXLINE];
	char	line2[MAXLINE];
	char	line3[MAXLINE];
	char	line4[MAXLINE];
	struct Entry	*next;
};


char *LoadFile( char *filename )
{
	FILE *FP;
	int size = 0;
	char *p;

	FP = fopen( filename, "rb" );
	if( !FP )
		return 0;
	fseek( FP, 0, SEEK_END );
	size = ftell( FP );
	fseek( FP, 0, SEEK_SET );
	p = (char*)malloc( size + 1 );
	memset( p, 0, size + 1 );
	fread( p, 1, size, FP );
	fclose( FP );
	return p;
}


struct Entry *AddEntry( struct Entry *e )
{
	struct Entry *n;

	if( e )
	{
		e->next = (struct Entry *)malloc( sizeof(struct Entry) );
		n = e->next;
	}
	else
		n = (struct Entry *)malloc( sizeof(struct Entry) );
				
	memset( n,0, sizeof( struct Entry ));
	return n;
}

int baudrate;

struct Entry *LoadEntries( char *filename )
{
	char *p;
	char *p2;
	char *p3;
	struct Entry *Entries;
	struct Entry *E2;
	char temp[64];

	p = LoadFile( filename );
	if( !p )
		return 0;

	p2 = strstr( p, "baudrate" );
	
	baudrate = 0;
	sscanf( p2,"baudrate=%d", &baudrate );	
	printf("Baudrate = %d\r\n",baudrate );

	E2 = Entries = AddEntry( 0 );

	p2 = p = strstr( p, "lcd" );
	while( p )
	{
		temp[0] = 0;
		sscanf( p, "lcd=%d",&Entries->LCD_number);
		p2 = strstr( p2, "line1" );
		p2 = strstr( p2, "\"");
		p2++;
		p3 = strstr( p2, "\"");
		*p3 = 0;
		strcpy( (char*)Entries->line1,p2);

		char *param = strstr(p,"cmd_attention");

		if( param )
		{
			sscanf( param,"cmd_attention=%d", &Entries->command_param );
			Entries->command = CMD_FADER;
		}
		param = strstr(p,"cmd_contrast");

		if( param )
		{
			sscanf( param,"cmd_contrast=%d", &Entries->command_param );
			Entries->command = CMD_CONTRAST;
		}

		*p3 = '\"';
		
		p2 = strstr( p2, "line2" );
		p2 = strstr( p2, "\"");
		p2++;
		p3 = strstr( p2, "\"");
		*p3 = 0;
		strcpy( (char*)Entries->line2,p2);
		*p3 = '"';
			
		p2 = strstr( p2, "line3" );
		p2 = strstr( p2, "\"");
		p2++;
		p3 = strstr( p2, "\"");
		*p3 = 0;
		strcpy( (char*)Entries->line3,p2);
		*p3 = '"';
			
		p2 = strstr( p2, "line4" );
		p2 = strstr( p2, "\"");
		p2++;
		p3 = strstr( p2, "\"");
		*p3 = 0;
		strcpy( (char*)Entries->line4,p2);
		*p3 = '"';
			
		p = strstr( p2, "lcd" );
		if( p )
			Entries = AddEntry( Entries );
	}
	
	return E2;
}


void delay( int t )
{

	for( ;t>0; t-- )
//		for( int t2=0 ;t2<10000; t2++ )	
//			for( int t3=0 ;t3<10000; t3++ )
				for( int t4=0 ;t4<10; t4++ );	
}

//============================================================================
int main(int argc, char* argv[])
{
	char temp[64];
	struct Entry *E;
	struct Entry *E2;

	printf("Loading Entries...\r\n");
	E = LoadEntries( "lcdloader.ini" );
    

  //Check for optional "clear" parameter and set flag if found
  int
    cleardisplay=0;

  int
    baud;

  baud=baudrate;

	
  E2 = E;


	printf("Starting traversing of loop\r\n");
	while( E2 )
	{
		sprintf(temp,"/dev/ttyS%d",E2->LCD_number );
		printf("### init serial : %s\r\n",temp);
		if(Serial_Init(temp,baud))
		{
			printf("Could not open port \"%s\" at \"%d\" baud.\n",temp,baud);
			E2 = E2->next;
			continue;
		}
		else
			printf("\"%s\" opened at \"%d\" baud.\n\n",temp,baud);

		printf("lcd = %d\r\n----------------------\r\n|%s|\r\n|%s|\r\n|%s|\r\n|%s|\r\n----------------------\r\n",E2->LCD_number,E2->line1,E2->line2,E2->line3,E2->line4);
		  //We will try to be tricky, and send out data that will show up correctly
		  //on both the 634 and 632.
		
		  //First turn off "scroll" and "wrap"
		  SendByte(20);
		  SendByte(24);
		 
		  //Write out the bottom two lines
		  //on the 634. The 632 will ignore
		  //these lines.
		
		  //Position to line 4, col 1
		  SendByte(17);
		  SendByte(0);
		  SendByte(3);
		    {
		                //12345678901234567890
		      SendString(E2->line4);
		    }
		
		  //Position to line 3, col 1
		  SendByte(17);
		  SendByte(0);
		  SendByte(2);
		    {
		                //12345678901234567890
		      SendString(E2->line3);
		    }
		

  //Write out top two lines of the 
  //634 and both lines of the 632.

  //Position to line 2, col 1
		  SendByte(17);
		  SendByte(0);
		  SendByte(1);
		    {
		                //12345678901234567890
		      SendString(E2->line2);
		    }
		
		  //Position to line 1, col 1
		  SendByte(17);
		  SendByte(0);
		  SendByte(0);
		    {
		                //12345678901234567890
		      SendString(E2->line1);
		    }
		
		if( E2->command == CMD_CONTRAST )
		{
			SendByte(15);	
			SendByte(E2->command_param);	
		}
		if( E2->command == CMD_FADER )
		{
			for(  int t2=0; t2< 2; t2++ )
			{			
				for( int t=0; t< 1000; t++ )
				{
					SendByte(14);	
					SendByte(t/10);	
					delay(10000);
				}
				for( int t=1000; t > 0; t-- )
				{
					SendByte(14);	
					SendByte(t/10);	
					delay(10000);
				}
			}
			for( int t=0; t< 1000; t++ )
			{
				SendByte(14);	
				SendByte(t/10);	
				delay(10000);
			}
		}
		printf("### uninit serial\r\n");
		Uninit_Serial();
		E2 = E2->next;
	}		
  
  if(cleardisplay==1) printf("Display Cleared.\n");
  printf("Done.\n\n");
  return 0;
  }
//============================================================================
